/*
 * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */
/*
 * Adapted from Java FX 2.2 Runtime sources by David Cooke, December 2014.
 */
package gamefx2.gpl.controls.behaviour;

import com.sun.javafx.Utils;
import com.sun.javafx.tk.Toolkit;
import javafx.event.EventType;
import javafx.scene.control.Control;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;

import static gamefx2.gpl.controls.behaviour.OptionalBoolEn.ANY;
import static gamefx2.gpl.controls.behaviour.OptionalBoolEn.FALSE;
import static gamefx2.gpl.controls.behaviour.OptionalBoolEn.TRUE;

/**
 * KeyBindings are used to describe which action should occur based on some
 * KeyEvent state and Control state. These bindings are used to populate the
 * keyBindings variable on BehaviorBase. The KeyBinding can be subclassed to add
 * additional matching criteria. A match in a subclass should always have a
 * specificity that is 1 greater than its superclass in the case of a match, or
 * 0 in the case where there is no match.
 *
 * Note that this API is, at present, quite odd in that you use a constructor
 * and then use shift(), ctrl(), alt(), or meta() separately. It gave me an
 * object-literal like approach but isn't ideal. We will want some builder
 * approach here (similar as in other places).
 *
 * @version $Revision: 1.3 $
 * @author David Cooke
 */
public class KeyBinding
{
    private final KeyCode	codeM;
    private EventType<KeyEvent> eventTypeM = KeyEvent.KEY_PRESSED;
    private final String	actionM;
    private OptionalBoolEn	shiftM = FALSE;
    private OptionalBoolEn	ctrlM = FALSE;
    private OptionalBoolEn	altM = FALSE;
    private OptionalBoolEn	metaM = FALSE;



    public KeyBinding(KeyCode code, String action)
    {
	codeM = code;
	actionM = action;
    }



    public KeyBinding(KeyCode code, EventType<KeyEvent> type, String action)
    {
	codeM = code;
	eventTypeM = type;
	actionM = action;
    }



    public KeyBinding shift()
    {
	return shift(TRUE);
    }



    public KeyBinding shift(OptionalBoolEn value)
    {
	shiftM = value;
	return this;
    }



    public KeyBinding ctrl()
    {
	return ctrl(TRUE);
    }



    public KeyBinding ctrl(OptionalBoolEn value)
    {
	ctrlM = value;
	return this;
    }



    public KeyBinding alt()
    {
	return alt(TRUE);
    }



    public KeyBinding alt(OptionalBoolEn value)
    {
	altM = value;
	return this;
    }



    public KeyBinding meta()
    {
	return meta(TRUE);
    }



    public KeyBinding meta(OptionalBoolEn value)
    {
	metaM = value;
	return this;
    }



    public KeyBinding shortcut()
    {
	if (Toolkit.getToolkit().getClass().getName().endsWith("StubToolkit"))
	{
            // FIXME: We've hit the terrible StubToolkit (which only appears 
	    // during testing). We will dumb down what we do here
	    if (Utils.isMac())
	    {
		return meta();
	    }
	    else
	    {
		return ctrl();
	    }
	}
	else
	{
	    switch (Toolkit.getToolkit().getPlatformShortcutKey())
	    {
		case SHIFT:
		    return shift();

		case CONTROL:
		    return ctrl();

		case ALT:
		    return alt();

		case META:
		    return meta();

		default:
		    return this;
	    }
	}
    }



    public final KeyCode getCode()
    {
	return codeM;
    }



    public final EventType<KeyEvent> getType()
    {
	return eventTypeM;
    }



    public final String getAction()
    {
	return actionM;
    }



    public final OptionalBoolEn getShift()
    {
	return shiftM;
    }



    public final OptionalBoolEn getCtrl()
    {
	return ctrlM;
    }



    public final OptionalBoolEn getAlt()
    {
	return altM;
    }



    public final OptionalBoolEn getMeta()
    {
	return metaM;
    }



    public int getSpecificity(Control control, KeyEvent event)
    {
	int spec;
	
	if (codeM != null && codeM != event.getCode())
	    return 0;
	else
	    spec = 1;
	
	if (!shiftM.equals(event.isShiftDown()))
	    return 0;
	else if (shiftM != ANY)
	    spec++;
	
	if (!ctrlM.equals(event.isControlDown()))
	    return 0;
	else if (ctrlM != ANY)
	    spec++;
	
	if (!altM.equals(event.isAltDown()))
	    return 0;
	else if (altM != ANY)
	    spec++;
	
	if (!metaM.equals(event.isMetaDown()))
	    return 0;
	else if (metaM != ANY)
	    spec++;
	
	if (eventTypeM != null && eventTypeM != event.getEventType())
	    return 0;
	else
	    spec++;
	
	// We can now trivially accept it
	return spec;
    }



    @Override
    public String toString()
    {
	return "KeyBinding [code=" + codeM + ", shift=" + shiftM
	    + ", ctrl=" + ctrlM + ", alt=" + altM
	    + ", meta=" + metaM + ", type=" + eventTypeM
	    + ", action=" + actionM + "]";
    }
}
